%% Beads sorter Based on excercise 4 of Coursera Machine Learning by Andrew Ng:

clear ; close all; clc

%%%%%%%% Input parameters %%%%%%%%%%%%%%%
% You may change these parameters according to your needs

% NN architecture.
input_layer_size  = 3    % 3 --> R,G,B input channels (called features)
hidden_layer_size = 8    % at 8, total bytes used by Arduino is 803 B only.
num_labels = 18          % 18 color labels, from 0 to 17
% number of training epochs. 1000 seems well enough.
niter = 1000;
% filename with raw data of R,G,B,C,Label. You may use your own data but keep format
datafilename = "pearlers_data.txt";
% folder for NN outputs. It will be created in main folder.
dirname = "bsNN_outputs\\";
% filename for the C code
cCodeFilename = "NN_code.c";

%%%%%%%%%%%% end %%%%%%%%%%%%%%%%%%%%%%%%

# Setup the parameters you will use for this exercise
dirname = sprintf("%s%f", dirname, time());  %to avoid overwriting previous results
mkdir(dirname);

# Load Training Data
fprintf('Loading data ...\n');
all_data = load(datafilename);

# get only white beads and calculate average RGB for them
whites = all_data(find(all_data(:,5) == 3), :);
whites = whites(:, 1:3);
whiteBeadAveRaw = mean(whites, 1);

# perform white balance on all data
whiteBalanceCoeffs = 1 ./ (whiteBeadAveRaw / mean(whiteBeadAveRaw))
all_data(:, 1:3) = all_data(:, 1:3) .* whiteBalanceCoeffs;

# split data into train and test sets: xvalid_data are not used.
mAll = size(all_data, 1);  #total number of available samples
[train_data, test_data, xvalid_data] = split_data(all_data, [0.6 0.4 0.0], num_labels);

# store copy of train/test data (they were chosen randomly from the datafilename)
save('train_data.txt', 'train_data')
movefile('train_data.txt', dirname);

save('test_data.txt', 'test_data')
movefile('test_data.txt', dirname);

# prepare input features (first three columns, i.e. RGB)
Xraw = train_data(:, 1:end-2);
# just display 10 samples
fprintf("Randomly selected 20 records of training data:\n");
ridx = randperm(size(Xraw,1), 10);

# prepare ground-truth (labels)
y = train_data(:, end) + 1;
[Xraw(ridx,:) y(ridx)]

# prapre test data
Xraw_t = test_data(:, 1:end-2);
yt = test_data(:, end) + 1;

# normalize features
[X, mu, sigma] = featureNormalize(Xraw);
[Xt, mu_t, sigma_t] = featureNormalize(Xraw_t);
m = size(X, 1)  #number of training samples

%% ================ Part 6: Initializing Pameters ================
fprintf('\nInitializing Neural Network Parameters ...\n')

initial_Theta1 = randInitializeWeights(input_layer_size, hidden_layer_size);
initial_Theta2 = randInitializeWeights(hidden_layer_size, num_labels);

% Unroll parameters
initial_nn_params = [initial_Theta1(:) ; initial_Theta2(:)];

nn_size = size(initial_nn_params, 1);

%% =================== Part 8: Training NN ===================
fprintf('\nTraining Neural Network... \n')
options = optimset('MaxIter', niter);

% we will not regularize NN
lambda = 0;

% Create "short hand" for the cost function to be minimized
costFunction = @(p) nnCostFunction(p, ...
                                   input_layer_size, ...
                                   hidden_layer_size, ...
                                   num_labels, X, y, lambda);

% Now, costFunction is a function that takes in only one argument (the
% neural network parameters)
[nn_params, cost] = fmincg(costFunction, initial_nn_params, options);

% Obtain Theta1 and Theta2 back from nn_params
Theta1 = reshape(nn_params(1:hidden_layer_size * (input_layer_size + 1)), ...
                 hidden_layer_size, (input_layer_size + 1));

Theta2 = reshape(nn_params((1 + (hidden_layer_size * (input_layer_size + 1))):end), ...
                 num_labels, (hidden_layer_size + 1));

# store results (for traceability only)
rfname = 'resulted_model.mat';
save(rfname, 'Theta1', 'Theta2', 'mu', 'sigma', 'input_layer_size', 'hidden_layer_size', 'num_labels', 'lambda', 'niter');
pause(0.25)

# generate C code for Arduino
T1 = single(Theta1)';
T2 = single(Theta2)';
TT  = [T1(:);T2(:)];
TT = typecast(TT, 'uint8'); # to keep full float precision
fprintf("Generating c code for Arduino\n");
generate_c_code_header(cCodeFilename, input_layer_size, hidden_layer_size, num_labels);
generate_c_code(cCodeFilename, 'NN_wav', whiteBeadAveRaw, 'float');
generate_c_code(cCodeFilename, 'NN_mu', mu, 'float');
generate_c_code(cCodeFilename, 'NN_sigma', sigma, 'float');
generate_c_code(cCodeFilename, 'NN_thetas', TT, 'uint8');
generate_c_code_footnote(cCodeFilename);

movefile(cCodeFilename, dirname);

%% ================= Part 10: Implement Predict =================
%  After training the neural network, we would like to use it to predict
%  the labels. You will now implement the "predict" function to use the
%  neural network to predict the labels of the training set. This lets
%  you compute the training set accuracy.

# prepare label names (for graphs)
xlabels = {"None"; "Pink"; "Red"; "White"; "Green"; "Blue"; "Yellow";
"Lt Green"; "Violet"; "Lt Violet"; "Orange"; "Lt Orange"; "Grey";
"Creme"; "Blue Sky"; "Lt Pink"; "Lt Yellow"; "Brown"};

J_test  = nnCostFunction(nn_params, input_layer_size, ...
                          hidden_layer_size, num_labels, Xt, yt, lambda);
fprintf("J_test = %f\n", J_test);

fsize = 6;  # font size for graph

# Create confusion matrix for trainig and test data
# h has size (m x num_labels) and presents probability for all labels (cols) at each test case (rows)
[pred, h] = predict(Theta1, Theta2, Xraw, mu, sigma, whiteBalanceCoeffs);
fprintf('Training Set Accuracy: %f\n', mean(double(pred == y)) * 100);
cm = confusionmat(pred, y);
fname = plotConfMatDark(cm, xlabels, fsize);
rename(fname, 'Conf_Mat_train.png');
movefile('Conf_Mat_train.png', dirname);

[pred, h] = predict(Theta1, Theta2, Xraw_t, mu, sigma, whiteBalanceCoeffs);
fprintf('Test Set Accuracy: %f\n', mean(double(pred == yt)) * 100);
cm = confusionmat(pred, yt);
fname = plotConfMatDark(cm, xlabels, fsize);
rename(fname, 'Conf_Mat_test.png');
movefile('Conf_Mat_test.png', dirname);

movefile(rfname, dirname);

fprintf('\nProgram end.\n');

